// 右键菜单
import React from 'react';
import styled from 'styled-components';
import Menu from './menu-base';
import { IContextMenuItem, IContextMenuUIItem, ITriggerContext } from './type';

const ContextMenuContainer = styled.div``;

export function typeOf(param: any) {
  return Object.prototype.toString.call(param).slice(8, -1).toLowerCase();
}

// TODO
const cloneDeep = (e: any) => {
  const dfs = (node: any) => {
    let children: any = null;
    if (node?.children?.length > 1) {
      children = node.children.map(dfs);
    }
    if (typeOf(node) === 'object') {
      if (children) {
        return { ...node, children };
      }
      return { ...node };
    } else if (typeOf(node) === 'array') {
      return [...node];
    }
    return node;
  };
  return e.map(dfs);
};

/**
 * 处理属性，
 *  1. 计算函数类型变为数值，例如disable等
 *  2. 删除不应该存在的变量
 *
 * @param menuList - 菜单列表
 * @param ctx - 上下文
 * @returns 菜单列表UI数据
 */
const computeMenuState = (
  menuList: IContextMenuItem[],
  ctx: ITriggerContext,
) => {
  const computeState = (menuItem: IContextMenuItem) => {
    // 除此之外的属性会别删除
    const arrowKeyMap = {
      text: true,
      key: true,
      tips: true,
      shortcutKeyDesc: true,
      disable: true,
      hide: true,
      icon: true,
      // 用户用来透传自定义数据的属性
      customData: true,
      children: true,
    };
    // 以下属性如果是函数会被运行处理成值
    const funcKeyMap: Record<string, boolean> = {
      text: true,
      disable: true,
      hide: true,
      icon: true,
      tips: true,
      children: true,
    };
    Object.keys(menuItem).forEach((key: string) => {
      if (!arrowKeyMap[key]) {
        delete menuItem[key];
        return;
      }
      if (funcKeyMap[key] && typeof menuItem?.[key] === 'function') {
        menuItem[key] = menuItem[key](ctx);
      }
    });
  };
  const dfs = (menuItem: IContextMenuItem) => {
    computeState(menuItem);
    if (menuItem?.children?.length > 0) {
      (menuItem?.children as IContextMenuItem[]).forEach(child => dfs(child));
    }
  };
  const newMenuList: IContextMenuUIItem[] = cloneDeep(menuList) as any;
  newMenuList.forEach(dfs);
  return newMenuList;
};

interface Point {
  x: number;
  y: number;
}

interface IProps {
  className?: string;
  style?: React.CSSProperties;
  menuList: IContextMenuItem[];
  getContainerDom: () => any;
  onTrigger?: (
    menuItem: IContextMenuUIItem,
    triggerContext: ITriggerContext,
  ) => void;
  onContextMenu?: (
    e: PointerEvent,
    triggerContext: ITriggerContext,
  ) => boolean | any;
  mergeContextFromEvent?: (params: {
    event: PointerEvent;
  }) => Record<string, any>;
  [key: string]: any;
}

interface IState {
  menuData: IContextMenuUIItem[];
  contextMenuPoint: Point;
  visiblePopover: boolean;
  triggerContext: ITriggerContext;
}
export class ContextMenu extends React.Component<IProps, IState> {
  menuContainerRef: any = React.createRef();

  containerDom: any = null;

  contextMenuEvent: PointerEvent;

  readonly state: IState = {
    contextMenuPoint: { x: 0, y: 0 },
    visiblePopover: false,
    triggerContext: { event: null as any },
    menuData: [],
  };

  componentDidMount() {
    setTimeout(() => {
      const { getContainerDom } = this.props;
      if (getContainerDom) {
        this.containerDom = getContainerDom();
      } else {
        this.containerDom = this.menuContainerRef.current?.parentNode;
      }
      this.containerDom?.addEventListener(
        'contextmenu',
        this.handleContextMenu,
      );
    }, 500);
  }

  componentWillUnmount() {
    this.containerDom?.removeEventListener(
      'contextmenu',
      this.handleContextMenu,
    );
  }

  handleContextMenu = (e: PointerEvent) => {
    const { menuList, onContextMenu, mergeContextFromEvent } = this.props;
    if (!this.containerDom) {
      return;
    }
    const moreContext = mergeContextFromEvent?.({ event: e }) || {};
    const newContext = {
      ...moreContext,
      event: e,
    };
    if (onContextMenu && onContextMenu(e, newContext) === false) {
      return;
    }
    this.hidePopover();
    // e.stopPropagation();
    e.preventDefault();
    const { x, y } = this.containerDom?.getBoundingClientRect();
    const point = { x: e.pageX - x, y: e.pageY - y };
    this.contextMenuEvent = e;
    // 计算状态
    const newMenuData = computeMenuState(menuList, newContext);
    this.showPopover(point);
    this.setState({
      triggerContext: newContext,
      menuData: newMenuData,
    });
  };

  handleTrigger = (menuItem: IContextMenuUIItem) => {
    const { onTrigger } = this.props;
    const { triggerContext } = this.state;
    onTrigger?.({ ...menuItem }, triggerContext);
  };

  showPopover = (point: Point) => {
    this.setState({ contextMenuPoint: point, visiblePopover: true });
  };

  hidePopover = () => {
    this.setState({ visiblePopover: false });
  };

  render() {
    const { menuData, contextMenuPoint, visiblePopover } = this.state;
    return (
      <ContextMenuContainer
        ref={this.menuContainerRef}
        style={{
          position: 'absolute',
          left: contextMenuPoint.x,
          top: contextMenuPoint.y,
        }}>
        <Menu
          {...this.props}
          menuList={menuData}
          visible={visiblePopover}
          onTrigger={this.handleTrigger}
          onVisibleChange={(visible: boolean) => {
            if (!visible) {
              this.hidePopover();
            }
          }}
        />
      </ContextMenuContainer>
    );
  }
}
