//@ts-ignore
import React, { useRef, useEffect } from 'react';
import styled from 'styled-components';
import { IContextMenuUIItem, ITriggerContext } from './type';
import { Tooltip } from './tips';

const CaretRightOutlined = (props: any) => <div {...props}>▶</div>;
const IconQuestionCircle = (props: any) => (
  <div
    {...props}
    style={{
      border: '1px solid #bbb',
      borderRadius: '50%',
      display: 'flex',
      alignItems: 'center',
      justifyContent: 'center',
      width: 14,
      height: 14,
      fontSize: 12,
      color: '#666',
      transform: 'scale(0.8)',
    }}>
    ?
  </div>
);

interface IProps {
  className?: string;
  style?: React.CSSProperties;
  visible?: boolean;
  menuList: IContextMenuUIItem[];
  onClick?: (e: MouseEvent) => void;
  onTrigger?: (menuItem: IContextMenuUIItem) => void;
  onVisibleChange?: (visible: boolean) => void;
}

const ContextMenuContainer = styled.div`
  box-shadow: 0 4px 10px #0001;
  .context-menu-mask {
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    opacity: 0.2;
    z-index: 1;
  }
  .context-menu-list {
    position: relative;
    min-width: 150px;
    max-width: 190px;
    font-size: 12px;
    line-height: 1;
    color: rgba(0, 0, 0, 0.88);
    user-select: none;
    padding: 5px 0;
    background-color: #fff;
    border-radius: 3px;
    border: 1px solid rgb(229, 230, 235);
    z-index: 10;
    .context-menu-item {
      position: relative;
      display: flex;
      align-items: center;
      padding: 6px 8px;
      cursor: pointer;
      height: 32px;
      box-sizing: border-box;
      position: relative;
      &:hover {
        background-color: #f6f6f6;
        & > .context-menu-list-children-container {
          display: block;
        }
      }
      .context-menu-icon {
        width: 18px;
        height: 18px;
        font-size: 14px;
        /* display: none; */
        display: flex;
        align-items: center;
        justify-content: center;
        margin-right: 2px;
      }
      .context-menu-text {
        overflow: hidden;
        text-overflow: ellipsis;
        white-space: nowrap;
      }
      .context-menu-tips-icon {
        margin-left: 2px;
        &:hover {
          color: #000;
        }
      }
      .context-menu-shortcut-key {
        margin-left: auto;
        color: #999;
        transform: scale(0.85);
      }
      .context-menu-more {
        margin-left: auto;
        color: #bbb;
        transform: scale(0.6, 0.8);
      }
    }
    .context-menu-item-disable {
      cursor: not-allowed;
      color: rgba(0, 0, 0, 0.38);
      .context-menu-list {
        display: none;
      }
      &:hover {
        background-color: inherit;
      }
    }
    .context-menu-item-hide {
      display: none;
    }
  }
  .context-menu-list-children-container {
    display: none;
    position: absolute;
    top: 0;
    right: calc(-100% - 10px);
    padding-left: 10px;
    & > .context-menu-list {
      box-shadow: 0 4px 10px #0001;
    }
  }
`;

const ContextMenuItemList = ({ children, className = '', ...rest }: any) => (
  <div className={`context-menu-list ${className}`} {...rest}>
    {children}
  </div>
);

const ContextMenuItem = ({ children, className = '', ...rest }: any) => (
  <div className={`context-menu-item ${className}`} {...rest}>
    {children}
  </div>
);

export default class MenuBase extends React.Component<any, any> {
  componentDidMount() {

    document?.addEventListener('click', this.handleClick);
  }
  componentWillUnMount() {

    document?.removeEventListener('click', this.handleClick);
  }

  handleClick = ()=> {
    const { onVisibleChange } = this.props || {};
    onVisibleChange?.(false);
  };
  render() {

    const {
      className = '',
      style = {},
      menuList = [],
      visible = false,
      onTrigger = () => '',
      onVisibleChange = () => '',
    } = this.props || {};
    const RenderMenuItem = ({ menu, showIcon = false } : any) => {
      const {
        text,
        icon,
        tips,
        disable,
        hide = false,
        shortcutKeyDesc,
        children,
      } = menu;
      let hideChildren = false;
      if (children?.filter(e => !e.hide).length === 0) {
        hideChildren = true;
      }
      const showChildIcon = Boolean(children?.find(e => e.icon));
      if (hide) {
        return null;
      }

      const childrenMenuList =
        children && !hideChildren ? (
          <div className="context-menu-list-children-container">
            <ContextMenuItemList>
              {children.map((childMenu: IContextMenuUIItem) => <RenderMenuItem key={childMenu.key} menu={childMenu} showIcon={showChildIcon} />)}
            </ContextMenuItemList>
          </div>
        ) : null;

      return (
        <ContextMenuItem
          key={menu.key}
          onClick={() => {
            if (disable || children) {
              return;
            }
            onTrigger(menu);
            onVisibleChange?.(false);
          }}
          className={`context-menu-item ${
            disable ? 'context-menu-item-disable' : ''
          } ${children ? 'popover-parent-container' : ''}`}>
          {/* {showIcon && <div className="context-menu-icon">{icon}</div>} */}
          <div className="context-menu-text" title={text}>
            {text}
          </div>
          {tips && (
            <Tooltip content={tips}>
              <IconQuestionCircle
                title={tips}
                className="context-menu-tips-icon"
              />
            </Tooltip>
          )}
          {shortcutKeyDesc && (
            <div className="context-menu-shortcut-key">{shortcutKeyDesc}</div>
          )}

          {children && (
            <div className="context-menu-more">
              <CaretRightOutlined />
            </div>
          )}
          {childrenMenuList}
        </ContextMenuItem>
      );
    };

    return (
      <ContextMenuContainer
        className={className}
        style={{ display: visible ? undefined : 'none', ...style }}
        onClick={(e: any) => e.stopPropagation()}>
        {/* <div className="context-menu-mask" onClick={() => onVisibleChange?.(false)} /> */}
        <ContextMenuItemList>
          {menuList.map((menu: any) => <RenderMenuItem key={menu?.key} menu={menu} showIcon={true} />)}
        </ContextMenuItemList>
      </ContextMenuContainer>
    );
  }
}

// export default (props: any) => {
//   const {
//     className = '',
//     style = {},
//     menuList = [],
//     visible = false,
//     onTrigger = () => '',
//     onVisibleChange = () => '',
//   } = props;
//   const RenderMenuItem = ({ menu, showIcon = false } : any) => {
//     const {
//       text,
//       icon,
//       tips,
//       disable,
//       hide = false,
//       shortcutKeyDesc,
//       children,
//     } = menu;
//     let hideChildren = false;
//     if (children?.filter(e => !e.hide).length === 0) {
//       hideChildren = true;
//     }
//     const showChildIcon = Boolean(children?.find(e => e.icon));

//     if (hide) {
//       return null;
//     }

//     const childrenMenuList =
//       children && !hideChildren ? (
//         <div className="context-menu-list-children-container">
//           <ContextMenuItemList>
//             {children.map((childMenu: IContextMenuUIItem) => <RenderMenuItem menu={childMen} showIcon={showChildIcon} />)
//             }
//           </ContextMenuItemList>
//         </div>
//       ) : null;

//     return (
//       <ContextMenuItem
//         key={menu.key}
//         onClick={() => {
//           if (disable || children) {
//             return;
//           }
//           onTrigger(menu);
//           onVisibleChange?.(false);
//         }}
//         className={`context-menu-item ${
//           disable ? 'context-menu-item-disable' : ''
//         } ${children ? 'popover-parent-container' : ''}`}>
//         {showIcon && <div className="context-menu-icon">{icon}</div>}
//         <div className="context-menu-text" title={text}>
//           {text}
//         </div>
//         {tips && (
//           <Tooltip content={tips}>
//             <IconQuestionCircle
//               title={tips}
//               className="context-menu-tips-icon"
//             />
//           </Tooltip>
//         )}
//         {shortcutKeyDesc && (
//           <div className="context-menu-shortcut-key">{shortcutKeyDesc}</div>
//         )}

//         {children && (
//           <div className="context-menu-more">
//             <CaretRightOutlined />
//           </div>
//         )}
//         {childrenMenuList}
//       </ContextMenuItem>
//     );
//   };

//   // // 点击其他地方，关闭
//   // useEffect(() => {
//   //   const handleClick = () => {
//   //     onVisibleChange?.(false);
//   //   };
//   //   document?.addEventListener('click', handleClick);
//   //   return () => {
//   //     document?.removeEventListener('click', handleClick);
//   //   };
//   // }, []);
//   return (
//     <ContextMenuContainer
//       className={className}
//       style={{ display: visible ? undefined : 'none', ...style }}
//       onClick={(e: any) => e.stopPropagation()}>
//       {/* <div className="context-menu-mask" onClick={() => onVisibleChange?.(false)} /> */}
//       <ContextMenuItemList>
//         {menuList.map((menu: any) => renderMenuItem(menu, true))}
//       </ContextMenuItemList>
//     </ContextMenuContainer>
//   );
// };
