/* eslint-disable max-statements */
import React, { useCallback, useRef, useState, useEffect } from 'react';
import styled from 'styled-components';
import {
  IconCaretDown,
  IconCaretUp,
  IconCopy,
  IconDelete,
  IconDragArrow,
  IconPaste,
  IconScissor,
} from '@arco-design/web-react/icon';
import { Message } from '@arco-design/web-react';
import { ContextMenuCommon } from '../../../utils/context-menu-common';
import { IContextMenuItem, ITriggerContext } from '../../../utils/context-menu-common/type';

interface IComp {
  id: string;
  props: any;
}

interface IContext extends ITriggerContext {
  activeComp: IComp;
  activeCompIndex: number;
  compLength: number;
  copyComp: IComp | null;
}

const cmd = 'Cmd';

const cloneDeep = (e: any) => JSON.parse(JSON.stringify(e));

// 菜单配置数据
const menuList: IContextMenuItem[] = [
  {
    text: '复制',
    key: 'copy',
    shortcutKeyDesc: `${cmd}+C`,
    icon: <IconCopy />,
    isDisable: (ctx: IContext) => !ctx.activeComp?.id,
  },
  {
    text: '粘贴',
    key: 'paste',
    icon: <IconPaste />,
    shortcutKeyDesc: `${cmd}+V`,
    isDisable: (ctx: IContext) => !ctx.copyComp?.id,
  },
  {
    text: '剪切',
    key: 'cut',
    icon: <IconScissor />,
    shortcutKeyDesc: `${cmd}+X`,
    isDisable: (ctx: IContext) => !ctx.activeComp?.id,
  },
  {
    text: '删除',
    key: 'remove',
    icon: <IconDelete />,
    shortcutKeyDesc: `Del`,
    isDisable: (ctx: IContext) => !ctx.activeComp?.id,
  },
  {
    text: '移动',
    tips: '上下移动组件',
    key: 'move',
    icon: <IconDragArrow />,
    isHide: (ctx: IContext) => {
      return ctx.activeCompIndex < 0;
    },
    children: [
      {
        text: '上移',
        key: 'move_up',
        icon: <IconCaretUp />,
        shortcutKeyDesc: `${cmd}+up`,
        isDisable: (ctx: IContext) => {
          return ctx.activeCompIndex <= 0;
        },
      },
      {
        text: '下移',
        key: 'move_down',
        icon: <IconCaretDown />,
        shortcutKeyDesc: `${cmd}+down`,
        isDisable: (ctx: IContext) => {
          return (
            ctx.activeCompIndex >= ctx.compLength - 1 || ctx.activeCompIndex < 0
          );
        },
      },
      {
        text: '置顶',
        key: 'move_top',
        icon: <IconCaretUp />,
        isDisable: (ctx: IContext) => {
          return ctx.activeCompIndex <= 0;
        },
      },
      {
        text: '置底',
        key: 'move_bottom',
        icon: <IconCaretDown />,
        isDisable: (ctx: IContext) => {
          return ctx.activeCompIndex >= ctx.compLength - 1 || ctx.activeCompIndex < 0
        },
      },
    ],
  },
];

const componentsList = [
  {
    id: 'comp-1',
    children: 'comp-1',
    props: { style: {} },
  },
  {
    id: 'comp-2',
    children: 'comp-2',
    props: { style: { backgroundColor: '#66f3', }, },
  },
  {
    id: 'comp-3',
    children: 'comp-3',
    props: { style: { backgroundColor: '#f663', }, },
  },
];

const CompItem = styled.div`
  padding: 20px;
  margin: 20px;
  background-color: #6f63;
  &.comp-item {
    &:hover {
      outline: 2px dashed #0067ed;
    }
  }
  &.comp-item-selected {
    outline: 2px solid #0067ed;
    &:hover {
      outline: 2px solid #0067ed;
    }
  }
`;

let clickTime = 0;

export default () => {
  const containerDomRef = React.useRef();
  const [compList, setCompList] = useState(componentsList);
  const [selectedComp, setSelectedComp] = useState(null);
  const copyCompJson = useRef(null);
  const handleContextMenu = (e: PointerEvent, ctx: IContext) => {
    console.info('menu open:', e, ctx);
  };
  const handleCopy = () => {
    copyCompJson.current = cloneDeep(selectedComp || null);
  };
  const handleCut = () => {
    copyCompJson.current = cloneDeep(selectedComp || null);

    if (!selectedComp) {
      return;
    }
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index !== -1) {
      const newCompList = [...compList];
      newCompList.splice(index, 1);
      setCompList(newCompList);
      setSelectedComp(null);
    }
  };
  const handlePaste = () => {
    const newComp = cloneDeep(copyCompJson.current || null);
    if (!newComp) {
      return;
    }
    newComp.id = `${Date.now()}`;
    newComp.children += `-${newComp?.id}`;
    setCompList([...compList, newComp]);
  };
  const handleRemove = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index !== -1) {
      const newCompList = [...compList];
      newCompList.splice(index, 1);
      setCompList(newCompList);
      setSelectedComp(null);
    }
  };
  const handleMoveUp = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index > 0) {
      const newCompList = [...compList];
      const pre = newCompList.splice(index, 1)[0];
      newCompList.splice(index - 1, 0, pre);
      setCompList(newCompList);
    }
  };
  const handleMoveDown = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index < compList.length - 1) {
      const newCompList = [...compList];
      const pre = newCompList.splice(index, 1)[0];
      newCompList.splice(index + 1, 0, pre);
      setCompList(newCompList);
    }
  };
  const handleMoveTop = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index > 0) {
      const newCompList = [...compList];
      const pre = newCompList.splice(index, 1)[0];
      newCompList.splice(0, 0, pre);
      setCompList(newCompList);
    }
  };
  const handleMoveBottom = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index < compList.length - 1) {
      const newCompList = [...compList];
      const pre = newCompList.splice(index, 1)[0];
      newCompList.splice(compList.length, 0, pre);
      setCompList(newCompList);
    }
  };
  const handleSelectUp = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index > 0) {
      setSelectedComp(compList[index - 1]);
    }
  };
  const handleSelectDown = () => {
    const index = compList.findIndex(e => e.id === selectedComp?.id);
    if (index < compList.length - 1) {
      setSelectedComp(compList[index + 1]);
    }
  };

  const handleTrigger = (key: string) => {
    if (key === 'copy') {
      handleCopy();
    } else if (key === 'cut') {
      handleCut();
    } else if (key === 'paste') {
      handlePaste();
    } else if (key === 'remove') {
      handleRemove();
    } else if (key === 'move_up') {
      handleMoveUp();
    } else if (key === 'move_down') {
      handleMoveDown();
    } else if (key === 'move_top') {
      handleMoveTop();
    }  else if (key === 'move_bottom') {
      handleMoveBottom();
    } else if (key === 'select_up') {
      handleSelectUp();
    } else if (key === 'select_down') {
      handleSelectDown();
    }
  };
  // 菜单点击触发
  const handleMenuTrigger = (menu: IContextMenuItem, ctx: IContext) => {
    console.info('menu trigger:', menu, ctx);
    Message.success(`点击菜单：${menu?.text as string}`);
    // 这里处理触发菜单后的逻辑....
    handleTrigger(menu.key);
  };
  const mergeContextFromEvent = useCallback(
    ({ event }: { event: PointerEvent }) => {
      const { target } = event as any;
      const id = target?.getAttribute('id');
      const index = compList.findIndex(e => e.id === id);
      const activeComp = compList.find(e => e.id === id) || null;
      return {
        id,
        activeComp,
        activeCompIndex: index,
        compLength: compList.length,
        copyComp: copyCompJson.current || null,
      };
    },
    [compList],
  );

  return (
    <div
      ref={containerDomRef}
      className="context-menu-test-div"
      onClick={() => {
        if (Date.now() - clickTime < 100) {
          return;
        }
        setSelectedComp(null)
      }}
      onContextMenu={() => setSelectedComp(null)}
      style={{
        position: 'relative',
        width: 600,
        minHeight: 600,
        padding: 20,
        margin: '0 auto',
        backgroundColor: '#f991',
      }}>
      <h2>右键菜单操作</h2>
      {compList.map(comp => (
        <CompItem
          className={`comp-item ${selectedComp?.id === comp?.id ? 'comp-item-selected' : ''}`}
          key={comp.id}
          id={comp.id}
          {...(comp.props || {})}
          onClick={(e: any) => {
            clickTime = Date.now();
            setSelectedComp(comp);
          }}
          onContextMenu={(e: any) => {
            e.stopPropagation();
            setSelectedComp(comp)
          }}>
          {comp?.children}
        </CompItem>
      ))}
      <ContextMenuCommon
        getContainerDom={() => containerDomRef.current}
        menuList={menuList}
        mergeContextFromEvent={mergeContextFromEvent}
        onContextMenu={handleContextMenu}
        onTrigger={handleMenuTrigger}
      />
    </div>
  );
};
